/*
 * Grid.java
 *
 * Created on January 9, 2007, 2:50 PM
 *
 */

package game.map;

import game.Main;
import game.entities.*;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author Kyle Florence
 */
public class Grid
{    
    private boolean drawGrid = true;			// Whether or not to draw the grid
    
    private static Image map;					// The Image representation of this grid
    private static Graphics2D g;				// The graphics context on which to generate the image
    private static Rectangle clip;				// The rectangular clip for the gameplay region
    
    private static int width;					// Grid Width
    private static int height;					// Grid Height
    private static int cellSize;				// The height and width of each individual Cell
    
    private static int minX;					// The lowest x-coordinate of the gameplay region
    private static int maxX;					// The highest x-coordinate of the gameplay region
    private static int minY;					// The lowest y-coordinate of the gameplay region
    private static int maxY;					// The highest y-coordinate of the gameplay region
    
    // Our list of each cell in this grid
    private static HashMap<Point, Cell> cellList;
    
    // Colors, will eventually be replaced with image tiles
    private static Color background  = Color.white;
    private static Color imageGrid   = Color.black;
    private static Color playable    = Color.lightGray;
    private static Color spawnable   = Color.green;
    private static Color exitable    = Color.red;

    // Default: 20x20 grid with 20x20 cells
    public Grid() { this(20, 20, 20); }
    
    // TODO: this will eventually be read from a file and composed of tiles
    // Creates a new instance of Grid
    public Grid(int n, int w, int h)
    {        
        // Temporary variables
        int r, c;
        Cell cell;
        
        width = w;		// The width of the grid
        height = h;		// The height of the grid
        cellSize = n;	// T width and height of the cells in the grid
        
    	// Create our cellList
    	cellList = new HashMap<Point, Cell>(w * h);
        
        // Set the minX and minY coordinates
        minX = ((Main.WIDTH - (w * cellSize)) / 2); 
        minY = ((Main.HEIGHT - (h * cellSize)) / 2);
        
        // Set the maxX and maxY coordinates based on grid and cell size
        maxX = (minX + (w * cellSize));
        maxY = (minY + (h * cellSize));
        
        // Set clip rectangle (based on a centered grid)
        clip = new Rectangle(minX, minY, (w * cellSize), (h * cellSize));
        
        // Fill in cells for this grid size
        // (Horizontally oriented grid)
        if (w >= h)
        {
            for (r = 1; r <= w; r++)
            {                
                for (c = 1; c <= h; c++)
                {                    
                    // Create new cell for this (row, column) location
                    cell = new Cell(r, c);
                    
                    cellList.put(cell.getGridLocation(), cell);
                    
                    // TESTING ONLY, REMOVE LATER
                    // Set up playable area with spawns and goals for testing
                    if ((r > (w / 2 - 5)) && (r <= (h / 2 + 5)))
                    {                      
                        // Make cell playable
                    	cell.togglePlayable();
                        
                        // Set spawns and exits
                        if (c == 1) cell.toggleSpawn();
                        if (c == height) cell.toggleGoal();
                    }
                }
            }
        }
        
        // Fill in cells for this grid size
        // (Vertically oriented grid)
        else
        {
            for (c = 1; c <= height; c++)
            {
                for (r = 1; r <= width; r++)
                {
                    // Create new cell for this (row, column) location
                    cell = new Cell(r, c);
                    cellList.put(cell.getGridLocation(), cell);
                    
                    // TESTING ONLY, REMOVE LATER
                    // Set up playable area with spawns and goals for testing
                    if ((r > (width / 2 - 5)) && (r <= (width / 2 + 5)))
                    {                        
                    	// Make cell playable
                    	cell.togglePlayable();
                        
                    	// Set spawns and exits
                        if (c == 1) cell.toggleSpawn();
                        if (c == height) cell.toggleGoal();
                    }
                }
            }           
        }   
        
        // For testing:
        cellList.get(new Point(6,3)).togglePlayable();
        cellList.get(new Point(7,3)).togglePlayable();
        cellList.get(new Point(8,3)).togglePlayable();
        cellList.get(new Point(9,3)).togglePlayable();
        cellList.get(new Point(10,3)).togglePlayable();
        cellList.get(new Point(11,3)).togglePlayable();
        cellList.get(new Point(12,3)).togglePlayable();
        cellList.get(new Point(13,3)).togglePlayable();
        cellList.get(new Point(14,3)).togglePlayable();
        
        cellList.get(new Point(7,5)).togglePlayable();
        cellList.get(new Point(7,6)).togglePlayable();
        cellList.get(new Point(7,7)).togglePlayable();
        cellList.get(new Point(7,8)).togglePlayable();
        cellList.get(new Point(7,9)).togglePlayable();
        cellList.get(new Point(7,10)).togglePlayable();
        cellList.get(new Point(7,11)).togglePlayable();
        cellList.get(new Point(7,12)).togglePlayable();
        
        cellList.get(new Point(8,12)).togglePlayable();
        cellList.get(new Point(9,12)).togglePlayable();
        cellList.get(new Point(10,12)).togglePlayable();
        cellList.get(new Point(11,12)).togglePlayable();
        cellList.get(new Point(12,12)).togglePlayable();
        cellList.get(new Point(13,12)).togglePlayable();
        cellList.get(new Point(14,12)).togglePlayable();
        cellList.get(new Point(15,12)).togglePlayable();
        
        // Draw the grid
        draw();
    }

    // Toggle the grid
    public void toggleGrid() {
    	drawGrid = !drawGrid;
    }
    
    // DRAW GRID/CELLS: this is only for testing
    // TODO: eventually the map will be generated in it's own 'Map' class.
    public void draw()
    {
        Cell cell;
        
        // Create an image of the grid, match width/height of game window
        map = Main.getGC().createCompatibleImage(Main.WIDTH, Main.HEIGHT);
        g = (Graphics2D) map.getGraphics();
        
        // Set clip
        g.setClip(clip);
        
        // Draw background color for playable area
        g.setColor(background);
        g.fill(clip);
        
        // Iterate through our cells
        Iterator<Point> it = cellList.keySet().iterator();
        	
        // Draw playable area
        while (it.hasNext())
        {        	   
            // Grab the next cell        		
            if ((cell = cellList.get(it.next())) != null)
            {                   
                // Fill the cell if it needs to be filled
                if (cell.isPlayable())
                {
                    // Set Color
              	    if (cell.isSpawn()) g.setColor(spawnable);
                    else if (cell.isGoal()) g.setColor(exitable);
                    else g.setColor(playable);
                	   
               	    // Fill cell
                    g.fill(cell.getBounds());
                }
        	}
        }     
        
        // Paint the grid
        if (drawGrid)
        {
            int i;
        	
            // Set grid color
	        g.setColor(imageGrid);
	        
	        // Draw vertical lines
	        for (i = 0; i <= width; i++)
	        	g.drawLine(minX + (cellSize * i), minY, minX + (cellSize * i), maxY);
	
	        // Draw horizontal lines
	        for (i = 0; i <= height; i++)
	            g.drawLine(minX,  minY + (cellSize * i), maxX, minY + (cellSize * i));
        }
        
        // Debug mode
        if (Main.inDebugMode())
        {	                            
            // Loop through entity's and draw their path and binding box
        	for (Entity entity : Main.getEntities())
        	{
        		// Set this entity's color
        		g.setColor(entity.getColor());
	            
	            // Draw this entity's path (if it has one)
        		if (entity instanceof AlienEntity)
        		{
    	            g.drawImage(entity.getPathImage(), 0, 0, null);
        		}
        	}
        }
        
        // Dispose of this graphics content
        g.dispose();
    }
    
    // Return this grid's cell size
    public static int getCellSize() {
        return cellSize;
    }
    
    // Return this grid's image
    public static Image mapImage() {
        return map;
    }
    
    // Return a cell from the grid (int row, int column)
    public static Cell getCell(int r, int c) {
    	return cellList.get(new Point(r, c));
    }
    
    // Return a cell from the grid (Point cell)
    public static Cell getCell(Point cell) {
    	return cellList.get(cell);
    }
  
    // Return the minX coordinate
    public static int getMinX() {
        return minX;
    }    
    
    // Return the minY coordinate
    public static int getMinY() {
        return minY;
    }
    
    // Return the maxX coordinate
    public static int getMaxX() {        
        return maxX;
    }
    
    // Return the maxY coordinate
    public static int getMaxY() {        
        return maxY;
    }
    
    // Return this grid's clipping rectangle
    public static Rectangle getClip() {
        return clip;
    }
    
    // Return the width of this grid in cells
    public static int getGridWidth() {
        return width;
    }
    
    // Return the height of this grid in cells
    public static int getGridHeight() {
        return height;
    }
    
    /**
     *
     * Useful functions
     *
     **/
    
    // Return Point(x, y) from Point(r, c)
    public static Point locationFromCell(Point cell) { 
        return locationFromCell(cell.x, cell.y); 
    }
    
    // Return Point(x, y) from (r, c)
    public static Point locationFromCell(int r, int c)
    {
    	// Make sure this point is within our grid
    	if (r <= width && c <= height)
    	{
	    	return new Point(
				(minX + ((r - 1) * cellSize)),
				(minY + ((c - 1) * cellSize))
			);
    	}
    	
    	// Otherwise, return Point(0, 0)
        return new Point();
    }
    
    // Return Point(r, c) from Point(x, y)
    public static Point cellFromLocation(Point loc) { 
        return cellFromLocation(loc.x, loc.y); 
    }
    
    // Return Point(r, c) from (x, y)
    public static Point cellFromLocation(int x, int y)
    {    	
    	// Make sure this point is within our grid
    	if ((x <= maxX && x >= minX) && (y <= maxY && y >= minY))
        {
	    	return new Point(
	            ((x - minX + cellSize) / cellSize), 
	            ((y - minY + cellSize) / cellSize)
	        );
        }

    	// Otherwise, return Point(0, 0)
        return new Point();
    }
}
